from werkzeug.routing import Rule

from canonical.landscape.api.account import (
    switch_account_handler,
    account_preferences_patch_handler,
    account_preferences_get_handler,
    max_attachment_size_put_handler,
    max_attachment_size_get_handler,
    max_people_count_put_handler,
    max_people_count_get_handler,
    salesforce_account_key_put_handler,
    salesforce_account_key_get_handler,
)
from canonical.landscape.api.activity import (
    activities_get_handler,
    activities_list_handler,
    activities_parents_list_handler,
    activities_revert,
    activities_reapply,
)
from canonical.landscape.api.administrator import administrator_put_handler
from canonical.landscape.api.alert import (
    alerts_list_handler,
    alerts_summary_list_handler,
    dismiss_active_alerts_handler,
)
from canonical.landscape.api.auth import login_post_handler, password_put_handler
from canonical.landscape.api.computer import (
    computers_activities_list_handler,
    computers_get_handler,
    computers_list_handler,
    computers_put_handler,
    computer_processes_list_handler,
    signal_computer_proc_post_handler,
    wsl_hosts_list_handler,
)
from canonical.landscape.api.distribution import wsl_instance_names
from canonical.landscape.api.eventlog import event_log_list_handler
from canonical.landscape.api.gpg import import_gpg_key_post_handler
from canonical.landscape.api.groups import (
    computer_groups_list_handler,
    computer_usergroups_update_bulk_handler,
    computer_user_groups_list_handler,
)
from canonical.landscape.api.invitation import (
    invitation_get_handler,
    invitation_list_handler,
    invitation_post_handler,
    invitation_delete_handler,
)
from canonical.landscape.api.package import (
    computer_packages_list_handler,
    packages_get_handler,
    computer_package_downgrades_get_handler,
    computer_packages_install_at_version_post_handler,
)
from canonical.landscape.api.packageprofile import (
    create_package_profile_constraints_handler,
    create_package_profile_handler,
    delete_package_profile_constraints_handler,
    get_computer_package_profile_handler,
    list_package_profile_constraints_handler,
    list_package_profiles_handler,
    parse_package_profile_material_handler,
    update_package_profile_handler,
    update_package_profile_constraint_handler,
)
from canonical.landscape.api.person import (
    api_credentials_get_handler,
    api_credentials_post_handler,
    person_get_handler,
    person_post_handler,
)
from canonical.landscape.api.repository.profile import (
    create_repository_profile_handler,
    list_repository_profiles_handler,
    update_repository_profile_handler,
)
from canonical.landscape.api.scripts import scripts_get_handler
from canonical.landscape.api.snap import (
    snaps_action_handler,
    installed_snaps_list_handler,
    available_snaps_get_handler,
    snap_info_get_handler,
)
from canonical.landscape.api.tag import tags_list_handler
from canonical.landscape.api.user import (
    create_user_post_handler,
    edit_user_handler,
    lock_user_post_handler,
    remove_user_handler,
    unlock_user_post_handler,
    users_get_handler,
)
from canonical.landscape.api.usns import (
    usns_get_handler,
    usns_list_handler,
    usns_remove_handler,
    usns_upgrade_handler,
)


DEFAULT_API_VERSION = "v2"

# Refer to https://werkzeug.palletsprojects.com/en/latest/routing for instructions
V2_API = [
    Rule("/activities", methods=["GET"], endpoint=activities_list_handler),
    Rule("/activities/<int:id>", methods=["GET"], endpoint=activities_get_handler),
    Rule(
        "/activities/parents", methods=["GET"], endpoint=activities_parents_list_handler
    ),
    Rule("/activities/reapply", methods=["POST"], endpoint=activities_reapply),
    Rule("/activities/revert", methods=["POST"], endpoint=activities_revert),
    Rule(
        "/administrators/<int:person_id>",
        methods=["PUT"],
        endpoint=administrator_put_handler,
    ),
    Rule("/alerts", methods=["GET"], endpoint=alerts_list_handler),
    Rule("/alerts/summary", methods=["GET"], endpoint=alerts_summary_list_handler),
    Rule(
        "/alerts/<int:alert_id>/dismiss",
        methods=["POST"],
        endpoint=dismiss_active_alerts_handler,
    ),
    Rule("/computers", methods=["GET"], endpoint=computers_list_handler),
    Rule(
        "/computers/<int:computer_id>",
        methods=["GET"],
        endpoint=computers_get_handler,
    ),
    Rule(
        "/computers/<int:computer_id>",
        methods=["PUT"],
        endpoint=computers_put_handler,
    ),
    Rule(
        "/computers/activities",
        methods=["GET"],
        endpoint=computers_activities_list_handler,
    ),
    Rule("/computers/wsl-hosts", methods=["GET"], endpoint=wsl_hosts_list_handler),
    Rule(
        "/computers/<int:computer_id>/groups",
        methods=["GET"],
        endpoint=computer_groups_list_handler,
    ),
    Rule(
        "/computers/<int:computer_id>/packages",
        methods=["GET"],
        endpoint=computer_packages_list_handler,
    ),
    Rule(
        "/computers/<int:computer_id>/packages/profile",
        methods=["GET"],
        endpoint=get_computer_package_profile_handler,
    ),
    Rule(
        "/computers/<int:computer_id>/snaps/installed",
        methods=["GET"],
        endpoint=installed_snaps_list_handler,
    ),
    Rule(
        "/computers/<int:computer_id>/snaps/available",
        methods=["GET"],
        endpoint=available_snaps_get_handler,
    ),
    Rule(
        "/computers/<int:computer_id>/snaps/<string:snap_name>/info",
        methods=["GET"],
        endpoint=snap_info_get_handler,
    ),
    Rule(
        "/computers/<int:computer_id>/users/<string:username>/groups",
        methods=["GET"],
        endpoint=computer_user_groups_list_handler,
    ),
    Rule(
        "/computers/<int:computer_id>/packages/installed/<string:package_name>/downgrades",
        methods=["GET"],
        endpoint=computer_package_downgrades_get_handler,
    ),
    Rule(
        "/computers/<int:computer_id>/packages/installed",
        methods=["POST"],
        endpoint=computer_packages_install_at_version_post_handler,
    ),
    Rule(
        "/computers/<int:computer_id>/processes",
        methods=["GET"],
        endpoint=computer_processes_list_handler,
    ),
    Rule(
        "/computers/<int:computer_id>/usergroups/update_bulk",
        methods=["POST"],
        endpoint=computer_usergroups_update_bulk_handler,
    ),
    Rule(
        "/computers/<int:computer_id>/usns/remove-packages",
        methods=["POST"],
        endpoint=usns_remove_handler,
    ),
    Rule(
        "/computers/<int:computer_id>/usns/upgrade-packages",
        methods=["POST"],
        endpoint=usns_upgrade_handler,
    ),
    Rule("/credentials", methods=["GET"], endpoint=api_credentials_get_handler),
    Rule("/credentials", methods=["POST"], endpoint=api_credentials_post_handler),
    Rule("/events", methods=["GET"], endpoint=event_log_list_handler),
    Rule("/gpg-key", methods=["POST"], endpoint=import_gpg_key_post_handler),
    Rule("/invitations", methods=["GET"], endpoint=invitation_list_handler),
    Rule("/invitations", methods=["POST"], endpoint=invitation_post_handler),
    Rule(
        "/invitations/<int:id>", methods=["DELETE"], endpoint=invitation_delete_handler
    ),
    Rule("/invitations/<int:id>", methods=["GET"], endpoint=invitation_get_handler),
    Rule("/login", methods=["POST"], endpoint=login_post_handler),
    Rule("/password", methods=["PUT"], endpoint=password_put_handler),
    Rule(
        "/packages/profile/parser",
        methods=["POST"],
        endpoint=parse_package_profile_material_handler,
    ),
    Rule("/packages", methods=["GET"], endpoint=packages_get_handler),
    Rule("/packageprofiles", methods=["GET"], endpoint=list_package_profiles_handler),
    Rule("/packageprofiles", methods=["POST"], endpoint=create_package_profile_handler),
    Rule(
        "/packageprofiles/<string:package_profile_name>",
        methods=["PUT"],
        endpoint=update_package_profile_handler,
    ),
    Rule(
        "/packageprofiles/<string:package_profile_name>/constraints",
        methods=["GET"],
        endpoint=list_package_profile_constraints_handler,
    ),
    Rule(
        "/packageprofiles/<string:package_profile_name>/constraints",
        methods=["POST"],
        endpoint=create_package_profile_constraints_handler,
    ),
    Rule(
        "/packageprofiles/<string:package_profile_name>/constraints",
        methods=["DELETE"],
        endpoint=delete_package_profile_constraints_handler,
    ),
    Rule(
        "/packageprofiles/<string:package_profile_name>/constraints/<int:id>",
        methods=["PUT"],
        endpoint=update_package_profile_constraint_handler,
    ),
    Rule("/person", methods=["GET"], endpoint=person_get_handler),
    Rule("/person", methods=["POST"], endpoint=person_post_handler),
    Rule("/preferences", methods=["GET"], endpoint=account_preferences_get_handler),
    Rule("/preferences", methods=["PATCH"], endpoint=account_preferences_patch_handler),
    Rule(
        "/max-attachment-size",
        methods=["GET"],
        endpoint=max_attachment_size_get_handler,
    ),
    Rule(
        "/max-attachment-size",
        methods=["PUT"],
        endpoint=max_attachment_size_put_handler,
    ),
    Rule("/max-people-count", methods=["GET"], endpoint=max_people_count_get_handler),
    Rule("/max-people-count", methods=["PUT"], endpoint=max_people_count_put_handler),
    Rule(
        "/salesforce-account-key",
        methods=["GET"],
        endpoint=salesforce_account_key_get_handler,
    ),
    Rule(
        "/salesforce-account-key",
        methods=["PUT"],
        endpoint=salesforce_account_key_put_handler,
    ),
    Rule(
        "/processes/<signal>",
        methods=["POST"],
        endpoint=signal_computer_proc_post_handler,
    ),
    Rule(
        "/repositoryprofiles",
        methods=["GET"],
        endpoint=list_repository_profiles_handler,
    ),
    Rule(
        "/repositoryprofiles",
        methods=["POST"],
        endpoint=create_repository_profile_handler,
    ),
    Rule(
        "/repositoryprofiles/<string:repository_profile_name>",
        methods=["PUT"],
        endpoint=update_repository_profile_handler,
    ),
    Rule("/scripts", methods=["GET"], endpoint=scripts_get_handler),
    Rule("/snaps", methods=["POST"], endpoint=snaps_action_handler),
    Rule("/switch-account", methods=["POST"], endpoint=switch_account_handler),
    Rule("/tags", methods=["GET"], endpoint=tags_list_handler),
    Rule("/users", methods=["DELETE"], endpoint=remove_user_handler),
    Rule("/users", methods=["GET"], endpoint=users_get_handler),
    Rule("/users", methods=["POST"], endpoint=create_user_post_handler),
    Rule("/users", methods=["PUT"], endpoint=edit_user_handler),
    Rule("/users/lock", methods=["POST"], endpoint=lock_user_post_handler),
    Rule("/users/unlock", methods=["POST"], endpoint=unlock_user_post_handler),
    Rule("/usns", methods=["GET"], endpoint=usns_list_handler),
    Rule("/usns/<usn_name>", methods=["GET"], endpoint=usns_get_handler),
    Rule("/wsl-instance-names", methods=["GET"], endpoint=wsl_instance_names),
]

APIS = {"v2": V2_API}
