# Copyright 2005-2013 Canonical Limited.  All rights reserved.

from canonical.landscape.api.secrets_client import get_secret
from canonical.landscape.model.action.handler import ActionHandler, action
from canonical.landscape.model.computer.management import ComputerManagement


@action
class CreateChildComputerActionHandler(ActionHandler):
    permission = "CreateComputer"

    def handle(
        self, computer_parent, computer_name, cloud_init=None, data_id=None, token=None
    ):
        if (not cloud_init) and data_id:
            secrets_name = computer_parent.secrets_name
            account_name = computer_parent.account.name
            cloud_init = get_secret(account_name, token, secrets_name, data_id)
        management = ComputerManagement(self.context, self.person)
        activity_parent = management.create_child_computer(
            self.person,
            computer_parent,
            computer_name,
            cloud_init,
        )

        return activity_parent.get_state()


@action
class DeleteChildComputersActionHandler(ActionHandler):
    permission = "RemoveComputer"

    def handle(self, computer_ids=None):
        if computer_ids is None:
            computer_ids = list(self.context.get_selected_computer_ids())
        management = ComputerManagement(self.context, self.person)
        activity_parent = management.delete_child_computers(self.person, computer_ids)

        return activity_parent.get_state()


@action
class RemoveComputersActionHandler(ActionHandler):
    permission = "RemoveComputer"

    def handle(self, computer_ids=None):
        selection = self.context.selection
        if computer_ids is None:
            computer_ids = list(selection.get_selected_computer_ids())
        self.send_job(
            account_id=self.context.account.id,
            person_id=self.person.id,
            computer_ids=computer_ids,
        )


@action
class SetDefaultChildComputerActionHandler(ActionHandler):
    permission = "EditComputerDetails"

    def handle(self, computer_child):
        management = ComputerManagement(self.context, self.person)
        activity_parent = management.set_default_child_computer(
            self.person, computer_child
        )

        return activity_parent.get_state()


@action
class ShutdownHostComputerActionHandler(ActionHandler):
    permission = "ShutdownComputer"

    def handle(self, host_computer):
        management = ComputerManagement(self.context, self.person)
        activity_parent = management.shutdown_host_computer(self.person, host_computer)

        return activity_parent.get_state()


@action
class StartChildComputersActionHandler(ActionHandler):
    permission = "RestartComputer"

    def handle(self, computer_ids=None):
        if computer_ids is None:
            computer_ids = list(self.context.get_selected_computer_ids())
        management = ComputerManagement(self.context, self.person)
        activity_parent = management.start_child_computers(self.person, computer_ids)

        return activity_parent.get_state()


@action
class StopChildComputersActionHandler(ActionHandler):
    permission = "ShutdownComputer"

    def handle(self, computer_ids=None):
        if computer_ids is None:
            computer_ids = list(self.context.get_selected_computer_ids())
        management = ComputerManagement(self.context, self.person)
        activity_parent = management.stop_child_computers(self.person, computer_ids)

        return activity_parent.get_state()
