"""Initial model to support associating users with computers.

Supports many-to-many relationships, although current usage is likely
only one person to many computers.

"""

from storm.locals import Storm, Int, Reference

from canonical.landscape.model.main.errors import AccountMismatchError


class PersonComputer(Storm):
    __storm_table__ = "person_computer"
    __storm_primary__ = "person_id", "computer_id"

    person_id = Int(allow_none=False)
    computer_id = Int(allow_none=False)

    person = Reference(person_id, "Person.id")
    computer = Reference(computer_id, "Computer.id")

    def __init__(self, person, computer):
        if computer.account not in list(person.accounts):
            raise AccountMismatchError(
                f"Computer account '{computer.account.name}' not in accounts for {person.name}"
            )
        self.person = person
        self.computer = computer
