# Copyright 2005-2024 Canonical Limited.  All rights reserved.

from zope.cachedescriptors.property import Lazy

from canonical.routes.publisher import get_route_object

from canonical.landscape.ui.sidebar.page import SidebarPage, FilteredSidebarPage


class KnowledgeBaseSidebarPage(SidebarPage):
    """A sidebar page for the Knowledge Base section."""

    @property
    def url(self) -> str:
        """Generates the canonical URL for the sidebar page."""
        if self.account:
            return f"{self.account.url}/kb{self.base_url}"
        else:
            return f"/kb{self.base_url}"


class KnowledgeBaseFilteredSidebarPage(FilteredSidebarPage):
    """A filtered sidebar page for the Knowledge Base section."""

    @Lazy
    def base_url(self):
        from canonical.landscape.model.knowledge.search import KnowledgeBase
        from canonical.landscape.ui.knowledge.routes import AccountKnowledgeBase

        context = get_route_object(AccountKnowledgeBase, self.request)
        if context is None:
            context = get_route_object(KnowledgeBase, self.request)

        return context.url
