# Copyright 2005-2022 Canonical Limited.  All rights reserved.

CSP_SOURCE_SELF = "'self'"
CSP_SOURCE_NONE = "'none'"
CSP_SOURCE_INLINE = "'unsafe-inline'"
CSP_SOURCE_EVAL = "'unsafe-eval'"
CSP_SOURCE_HASH = "'unsafe-hashes'"
CSP_SOURCE_ASSETS = "assets.ubuntu.com"
CSP_SOURCE_GTM = (
    "www.googletagmanager.com www.google-analytics.com "
    "script.crazyegg.com www.google.com www.google.ca"
)
CSP_SOURCE_MAZE = "https://*.maze.co/"
CSP_KIND_DEFAULT = "default-src"
CSP_KIND_SCRIPT = "script-src"
CSP_KIND_STYLE = "style-src"
CSP_KIND_IMG = "img-src"
CSP_KIND_FONT = "font-src"
CSP_KIND_CONNECT = "connect-src"
# default content security policies in the form: {kind: (source,),}
DEFAULT_CSP = {
    CSP_KIND_SCRIPT: (
        CSP_SOURCE_SELF,
        CSP_SOURCE_INLINE,
        CSP_SOURCE_EVAL,  # required for YUI
        CSP_SOURCE_ASSETS,
        CSP_SOURCE_GTM,
        CSP_SOURCE_MAZE,
    ),  # required for Google Tag Manager
    CSP_KIND_STYLE: (
        CSP_SOURCE_SELF,
        CSP_SOURCE_INLINE,
        CSP_SOURCE_ASSETS,
        CSP_SOURCE_MAZE,
    ),
    CSP_KIND_IMG: (
        CSP_SOURCE_SELF,
        CSP_SOURCE_ASSETS,  # billing and front-page resources
        "data:",  # external link hint icon
        CSP_SOURCE_GTM,
        CSP_SOURCE_MAZE,
    ),
    CSP_KIND_FONT: (
        CSP_SOURCE_SELF,
        CSP_SOURCE_MAZE,
    ),
    CSP_KIND_CONNECT: (
        CSP_SOURCE_SELF,
        CSP_SOURCE_MAZE,
    ),
}


def make_csp_header(kind_sources, inherit=DEFAULT_CSP):
    """Build the page content-security-policy header from a map of sources.

    @param kind_sources: a map of {csp_kind: (source,)}
    @param inherit: a policy map to extend, or None.
    """
    policies = (inherit or {}).copy()
    policies.update((k, policies.get(k, ()) + v) for k, v in kind_sources.items())

    return ";".join(["{} {}".format(k, " ".join(v)) for k, v in policies.items()])
